<?php
/**
 * GForge Plugin gitTracker Class
 *
 * Portions Copyright 2012 (c) Song Xiuyao <songxy @nospam@ ufl.edu>
 * Portions Copyright 2012 (c) Shanan Smith <scsmith @nospam@ ufl.edu>
 * Copyright 2004 (c) Francisco Gimeno <kikov @nospam@ kikov.org>
 * Copyright 2005 (c) Guillaume Smet <guillaume-gforge@smet.org>
 * Copyright (C) 2010 Alain Peyrat - Alcatel-Lucent
 *
 * This file is part of GForge-plugin-gittracker
 *
 * GForge-plugin-gittracker is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * GForge-plugin-gittracker is distributed in the hope that it will
 * be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
/**
 * The gittrackerPlugin class. It implements the Hooks for the presentation
 *  of table in tracker and task in detailed mode.
 *
 */
require_once $gfcommon.'include/scm.php';

class gittrackerPlugin extends Plugin {
	function gittrackerPlugin () {
		$this->Plugin() ;
		$this->name = "gittracker" ;
		$this->text = "git<->Tracker";
		$this->hooks[] = "groupisactivecheckbox";
		$this->hooks[] = "groupisactivecheckboxpost";
		$this->hooks[] = "artifact_extra_detail";
		$this->hooks[] = "task_extra_detail";
		$this->hooks[] = "update_git_repository";
		$this->hooks[] = "get_git_loginfo_lines";
		$this->hooks[] = "addGitHookUpdate";
		

	}

	function groupisactivecheckbox (&$params) {
		$group = group_get_object($params['group']);
		if ($group->usesPlugin('scmgit')) {
			parent::groupisactivecheckbox($params);
		}
	}

	/**
	* It display a table with commit related to this tracker or task_extra_detail
	*
	* @param   string   $Query Query to be executed to get the commit entries.
	* @param   integer  $group_id Group_id of the actual Group_id
	*
	*/
	function getCommitEntries($DBResult,$group_id) {
		$group = group_get_object($group_id);
		
		if (!$group->usesPlugin($this->name)) {
			return;
		}
		
		$Rows= db_numrows($DBResult);
			
		if ($Rows > 0) {
			echo '<tr><td colspan="2">';
			echo '<h4>'._('Related git commits').':</h4>';

			$title_arr=$this->getTitleArr();
			echo $GLOBALS['HTML']->listTableTop ($title_arr);
		
			for ($i=0; $i<$Rows; $i++) {
				$Row = db_fetch_array($DBResult);
				echo '<tr '. $GLOBALS['HTML']->boxGetAltRowStyle($i) .'>'.
				//'<td>'. $this->getFileLink($group->getUnixName(),
				//		$Row['file'],$Row['actual_version']). '</td>'.
				'<td>'. date('m/d/Y', strtotime($Row['git_date'])).'</td>'.
				'<td>'. $this->getDiffLink($group->getUnixName(),
						$Row['file'],
						$Row['prev_version'],
						$Row['actual_version']).'</td>'.
				'<td>'. $this->getActualVersionLink($group->getUnixName(),
					$Row['file'], $Row['actual_version']).
				'</td>
				<td>'. htmlspecialchars($Row['log_text']).'</td>
				<td>'. util_make_link_u ($Row['author'],
							 user_get_object_by_name ($Row['author'])->getId(),
							 $Row['author']).'</td>
				</tr>';
			}
			echo $GLOBALS['HTML']->listTableBottom();
			echo '</td></tr>';
		} else {
			echo '<h4>'._('No commits have been made.').'</h4>';
		}
	}

	/**
	* Return an array with titles of Box to display the entries
	*
	* @return   Array  The array containing the titles
	*
	*/
	function getTitleArr() {
		$title_arr=array();
		//$title_arr[]=_('Filename');
		$title_arr[]=_('Date');
		$title_arr[]=_('Previous Version');
		$title_arr[]=_('Current Version');
		$title_arr[]=_('Log Message');
		$title_arr[]=_('Author');
		return $title_arr;
	}

	/**
	* Return a link to the File in gitweb
	*
	* @param    String  $GroupName is the Name of the project
	* @param    String  $FileName  is the FileName ( with path )
	*
	* @return   String  The string containing a link to the File in the gitweb
	*
	*/
	function getFileLink($GroupName, $FileName, $LatestRevision) {
		//return utiaddGitHookUpdatel_make_link ('/scm/viewvc.php/'.$FileName .
		//		      '?root='.$GroupName.'&amp;view=log',
		//		      $FileName);
		return $FileName;
	}

	/**
	* Return a link to the File in gitweb in the specified Version
	*
	* @param    String  $GroupName is the Name of the project
	* @param    String  $FileName  is the FileName ( with path )
	* @param    String  $Version   the version to retrieve
	*
	* @return   String  The string containing a link to the File in the gitweb
	*
	*/
	function getActualVersionLink($GroupName, $FileName, $Version) {
		//return $Version;
		return substr($Version, 0, 7);
		/*return util_make_link ('/scm/viewvc.php/'.$FileName .
				      '?root='.$GroupName.'&pathrev='.$Version .
				      '&amp;view=markup',
				      $Version);*/
	}

	/**
	* Return a link to the diff between two versions of a File in gitweb
	*
	* @param    String  $GroupName is the Name of the project
	* @param    String  $FileName  is the FileName ( with path )
	* @param    String  $PrevVersion   First version to retrieve
	* @param    String  $ActualVersion Second version to retrieve
	*
	* @return   String  The string containing a link to the File in the gitweb
	*update_git_repository
	*/
	function getDiffLink($GroupName, $FileName, $PrevVersion, $ActualVersion) {
		//return $PrevVersion;
		return substr($PrevVersion, 0, 7);
		/*return util_make_link ('/scm/viewvc.php/'.$FileName .
				      '?root='.$GroupName.'&r1='.$PrevVersion .
				      '&r2='.$ActualVersion,
				      _('Diff To').' '.$PrevVersion);*/
	}

	/**
	* The function to be called for a Hook
	*
	* @param    String  $hookname  The name of the hookname that has been happened
	* @param    String  $params    The params of the Hook
	*
	*/
	function CallHook ($hookname, &$params) {
		global $group_id, $G_SESSION, $HTML, $aid ;

		$FOut = fopen('/var/lib/gforge/tmp/gittracker_logfile', "a");
		fwrite($FOut, "\n gittrackerPlugin.class.php   CallHook() \n");
		fwrite($FOut, "\$hookname: $hookname");
		fclose($FOut);
		
		if ($hookname == "groupisactivecheckbox") {
			//Check if the group is active
			$group = &group_get_object($group_id);
			if ($group->usesPlugin('scmgit')) {
				echo "<tr>";
				echo "<td>";
				echo ' <input type="checkbox" name="use_gittrackerplugin4343s" value="1" ';
				// checked or unchecked?
				if ( $group->usesPlugin ( $this->name ) ) {
					echo "checked";
				}
				echo "><br/>";
				echo "</td>";
				echo "<td>";
				echo "<strong>Use ".$this->text." Plugin</strong>";
				echo "</td>";
				echo "</tr>";
			}
		} elseif ($hookname == "groupisactivecheckboxpost") {
			$group = &group_get_object($group_id);
			if ( $use_gittrackerplugin == 1 ) {
				$group->setPluginUse ( $this->name );
			} else {
				$group->setPluginUse ( $this->name, false );
			}
		} elseif ($hookname == "addGitHookUpdate") {
			//add a git hook to the git repo for this project
			$this->githookinstall($group_id);
			
		} elseif ($hookname == "artifact_extra_detail") {
			$DBResult = db_query_params ('SELECT * FROM plugin_gittracker_data_master,plugin_gittracker_data_artifact WHERE plugin_gittracker_data_artifact.group_artifact_id=$1 AND plugin_gittracker_data_master.holder_id=plugin_gittracker_data_artifact.id ORDER BY git_date',
				array ($aid)) ;
			
			$this->getCommitEntries($DBResult, $group_id);
		} elseif ($hookname == "task_extra_detail") {
			$DBResult = db_query_params ('SELECT * FROM plugin_gittracker_data_master,plugin_gittracker_data_artifact WHERE plugin_gittracker_data_artifact.project_task_id=$1 AND plugin_gittracker_data_master.holder_id=plugin_gittracker_data_artifact.id ORDER BY git_date',
				array ($params['task_id'])) ;
			$this->getCommitEntries($DBResult, $group_id);
		} elseif ($hookname == "update_git_repository") {
			
			$Group = group_get_object($params["group_id"]);
			
			if ($Group->usesPlugin("gittracker")) {
				$LineFound=FALSE;
				$FIn = fopen(getgitFile( $params["file_name"],
					"gitROOT/loginfo"),"r");
				if ($FIn) {
					while (!feof($FIn))  {
						$Line = fgets ($FIn);
						if(!preg_match("/^#/", $Line) &&
							preg_match("/gittracker/",$Line)) {
								$LineFound = TRUE;
							}
				 	}
				 	fclose($FIn);
				}				
				if($LineFound==FALSE) {
					$newfile=getgitFile($params["file_name"],
						 "gitROOT/loginfo");
					$this->addgitTrackerToFile($Group, $newfile);
					$this->putgitFile($params["file_name"],
						$newfile);
				}
			}
		} elseif ($hookname == "get_git_loginfo_lines") {
			$group = group_get_object($group_id);
			$GLOBALS['loginfo_lines']=$this->getgitTrackerLogInfoLines($group);		
		}
	}
	
	function githookinstall($group_id)
	{
		$project = group_get_object($group_id);
		if (!$project) {
			return false ;
		}
		
		if (! $project->usesPlugin ($this->name)) {
			return false;
		}
		
		$project_name = $project->getUnixName() ;
		
		//this is for testing on a local machoine with no ssl certs
		if(forge_get_config('web_host') == 'forgedev'){
			$baseLocation = 'http://'.forge_get_config('web_host');
		}else{
			$baseLocation = util_url_prefix().forge_get_config('web_host');
		}
		
		$root = forge_get_config('repos_path', 'scmgit') . '/' . $project_name ;
		$unix_group = 'scm_' . $project_name ;
		
		$main_repo = $root . '/' .  $project_name . '.git' ;
			
		$update_location = $main_repo.'/hooks/update';
		
		if (is_file ($main_repo.'/HEAD') && is_dir ($main_repo.'/objects') && is_dir ($main_repo.'/refs')) {
				
			if (!is_file($update_location) || filesize($update_location) == 0) {
				$createFile = fopen($update_location, "w");
				
				fwrite($createFile, "#!/bin/sh \n");
				fwrite($createFile, "# \n");
				fwrite($createFile, "# --- Command line \n");
				fwrite($createFile, "refname=\"$1\" \n");
				fwrite($createFile, "oldrev=\"$2\" \n");
				fwrite($createFile, "newrev=\"$3\" \n");
				
				fclose($createFile) ;
			}
			
			$needUpdateFlag = 1;
			$f = fopen ($update_location, "r");
			while (!feof($f)) {
				$line = fgets($f);
				if (strpos($line, "gittracker/newpost.php")){
					$needUpdateFlag = 0;
				}
			}
			fclose ($f) ;
			
			if($needUpdateFlag){
				$codeAddedFlag = 0;
				$f = fopen ($update_location, "r");
				$createFile = fopen($update_location.'.tmp', "w");
				while (!feof($f)) {
					$line = fgets($f);
					if (substr($line, 0, 1) != "#" && $codeAddedFlag == 0){
						$codeAddedFlag = 1;
						fwrite ($createFile, " \n# This code is used in the gittracker system for fusionforgey \n ") ;
						fwrite ($createFile, " \nuser=`whoami` \n") ;
						fwrite ($createFile, "allcommits=`git log $2..$3` \n") ;
						fwrite ($createFile, "if [ $(git rev-parse --is-bare-repository) = true ] \n") ;
						fwrite ($createFile, "then \n") ;
						fwrite ($createFile, "REPOSITORY_BASENAME=$(basename \"\$PWD\") \n") ;
						fwrite ($createFile, "else \n") ;
						fwrite ($createFile, "	REPOSITORY_BASENAME=$(basename $(readlink -nf \"\$PWD\"/..)) \n") ;
						fwrite ($createFile, "	fi \n") ;
						fwrite ($createFile, "REPOSITORY_BASENAME=\${REPOSITORY_BASENAME%.git} \n") ;
						fwrite ($createFile, "`curl -d \"user=\$user&comments=\$allcommits&repoName=\$REPOSITORY_BASENAME&oldrev=$2\"  $baseLocation/plugins/gittracker/newpost.php` \n") ;
						fwrite ($createFile, " \n \n \n") ;
					}
					
					fwrite ($createFile, $line);
					
				}
				fclose($createFile) ;
				fclose ($f) ;
				system ('mv '.$update_location.'.tmp '.$update_location) ;
				
				system ('chgrp '.$unix_group.' '.$update_location) ;
				system ('chmod 770 '.$update_location) ;
			}
		}
		echo util_url_prefix();
		echo "\n";
	}
}


// Local Variables:
// mode: php
// c-file-style: "bsd"
// End:

?>
