<?php 
/**
  * 
  * gittracker plugin
  *
  * This script stand for receiving a HTTP Post from a git hook.
  * Arguments are passed and new entries in DB are generated.
  *
  * Copyright Francisco Gimeno <kikov@kikov.org>
  * Copyright 2012 (c) Song Xiuyao <songxy @nospam@ ufl.edu>
  * Copyright 2012 (c) Shanan Smith <scsmith @nospam@ ufl.edu>
  */
/**
 * This script takes some POST variables.It makes a check and it
 * store in DB the commit info attached to the tracker or task.
 *
 */

require_once dirname(__FILE__)."/../../env.inc.php";
require_once $gfcommon.'include/pre.php';

/**
 * Getting POST variables
 * UserName, Repository, Path, FileName, PrevVersion, ActualVersion, Log,
 * ArtifactNumbers and TaskNumbers
 */
$Config = array();
$SubmittedVars = array();

//pulling data from the git hook "update"
$allCommits = $_POST['comments'];
$repoName = $_POST['repoName'];
$oldrev = $_POST['oldrev'];
$UserName = $_POST['user'];

$SubmittedVars = commitSplitter($allCommits, $repoName);

$i = 0;
foreach ($SubmittedVars as $SubmittedVar) {
	$Configs[$i]['UserName']        = $UserName;
	$Configs[$i]['Repository']      = $SubmittedVar['Repository'];
	//$Configs[$i]['FileName']        = $SubmittedVar['FileName'];
	$Configs[$i]['PrevVersion']     = $SubmittedVar['PrevVersion'];
	$Configs[$i]['ActualVersion']   = $SubmittedVar['ActualVersion'];
	$Configs[$i]['ArtifactNumbers'] = getInvolvedArtifacts($SubmittedVar['Log']);
	$Configs[$i]['TaskNumbers']     = getInvolvedTasks($SubmittedVar['Log']);
	$Configs[$i]['Log']             = $SubmittedVar['Log'];
	$Configs[$i]['gitDate']         = $SubmittedVar['gitDate'];
	$i++;
}


foreach ($Configs as $Config) {
	$Result = parseConfig($Config);

	if ($Result['check'] == false) {
		exit_error('Check_error', $Result['error']);
	}

	if (!is_null($Config['ArtifactNumbers'])) {
		foreach ($Config['ArtifactNumbers'] as $Num)
		{
			$AddResult = addArtifactLog($Config, $Result['group_id'], $Num);
			if (isset($AddResult['Error'])) {
				exit_error('Adding ArtifactNumber',$AddResult['Error']);
			}
		}
	}

	if (!is_null($Config['TaskNumbers'])) {
		foreach ($Config['TaskNumbers'] as $Num)
		{
			$AddResult = addTaskLog($Config, $Result['group_id'], $Num);
			if (isset($AddResult['Error'])) {
				exit_error('Adding TaskNumber',$AddResult['Error']);
			}
		}
	}
}

/**
 * 
 * Take the git logs information and split them into a multidimensional array.
 * 
 * @param unknown_type $allCommits
 * @param unknown_type $repoName
 * @return multitype:
 * @author scsmith songxy
 */
function commitSplitter ($allCommits=array(), $repoName){
	$output = array();
	$output = explode("\n", $allCommits);
	
	$history = array();
	$commit = array();
	foreach($output as $line){
		$pattern = "/^commit\ [a-z0-9]{40}/";
		if(preg_match($pattern, $line)){
			if(!empty($commit)){
				$commit['PrevVersion'] = trim(substr($line, strlen('commit')));
				array_push($history, $commit);
				unset($commit);
			}
			$commit['ActualVersion'] = trim(substr($line, strlen('commit')));
			$commit['Repository'] = $repoName;
		}
		else if(strpos($line, 'Author')===0){
			$commit['UserName'] = trim(htmlspecialchars(substr($line, strlen('Author:'))));
		}
		else if(strpos($line, 'Date')===0){
			$commit['gitDate'] = trim(substr($line, strlen('Date:')));
		}
		else{
			$commit['Log'] .= trim(htmlspecialchars($line));
		}
	}
	$commit['PrevVersion'] = trim(substr($line, strlen('commit')));
	array_push($history, $commit);
	
	return $history;
}

/**
 * It returns a list of involved artifacts.
 * An artifact is identified if [#(NUMBER)] if found.
 *
 * @param   string   $Log Log message to be parsed.
 *
 * @return  boot    Returns true if check passed.
 */
function getInvolvedArtifacts($Log)
{
	preg_match_all('/[[]#[\d]+[]]/', $Log,  $Matches );
	foreach($Matches as $Match)
	{
		$Result = preg_replace ('/[[]#([\d]+)[]]/', '\1', $Match);
		
	}
	return $Result;
}

/**
 * It returns a list of involved artifacts.
 * An artifact is identified if [T(NUMBER)] is found.
 *
 * @param   string   $Log Log message to be parsed.
 *
 * @return  boot    Returns true if check passed.
 */
function getInvolvedTasks($Log)
{	
	preg_match_all ('/[[]T[\d]+[]]/', $Log,  $Matches );
	foreach($Matches as $Match)
	{
		$Result = preg_replace ('/[[]T([\d]+)[]]/', '\1', $Match);
	}	
	return $Result;
}

/**
 * Checks if the commit it's possible and parse arguments
 * Checks if repository, group and user_name are right.
 *  It extract group from gitroot, and check if the plugin
 *  is availabe. It checks if the user exists.
 *
 * @param   array    $Config Config
 *
 * @return  array    Returns 'check'=true if check passed, group, group_id
 */
function parseConfig($Config)
{
	$repos_path = forge_get_config ('repos_path', 'scmgit') ;
	
	$Result = array();
	$Result['check'] = true;
	$Repository = $Config['Repository'];
	$UserName = $Config['UserName'];

	// add a trailing / if needed
	if($repos_path[strlen($repos_path)-1]!='/') {
		$repos_path.='/';
	}
	if(strncmp($Repository,$repos_path, strlen($repos_path)) == 0) {
		$GroupName = substr($Repository, strlen($repos_path));
	} else {
		$GroupName = $Repository;
	}
	
	$Result['user'] = user_get_object_by_name($UserName);
	if (!$Result['user'] || !is_object($Result['user']) ||
		$Result['user']->isError() || !$Result['user']->isActive()) {
		$Result['check'] = false;
		$Result['error'] = 'Invalid User';
		return $Result;
	}
	session_set_new($Result['user']->getID());

	$Result['group']    = group_get_object_by_name($GroupName);
	if (!$Result['group'] || !is_object($Result['group']) ||
		$Result['group']->isError() || !$Result['group']->isActive()) {
		$Result['check'] = false;
		$Result['error'] = 'Group Not Found';
	} else {
		$Result['group_id'] = $Result['group']->getID();
		if (!$Result['group']->usesPlugin('gittracker')) {
			$Result['check'] = false;
			$Result['error'] = 'Plugin not enabled for this Group';
		}
	}

	return $Result;
}

/**
 * Add a entry in the DataBase for a Artifact associated to a commit
 *
 * @param   array    $Config Config
 * @param   string 	$GroupId The GroupId to insert it into
 * @param   string   $Num The artifact_id
 *
 * @return  array    Returns 'check'=true if check passed, group, group_id
 */
function addArtifactLog($Config, $GroupId, $Num)
{
	$return = array();
	$Result = db_query_params ('SELECT * from artifact,artifact_group_list WHERE artifact.group_artifact_id=artifact_group_list.group_artifact_id AND artifact_group_list.group_id=$1 AND artifact.artifact_id=$2',
		array ($GroupId, $Num));
	
	$Rows = db_numrows($Result);
	
	if ($Rows == 0) {
		$return['Error'] .= "Artifact ".$Num." Not Found.";
	}

	if ($Rows == 1) {
		db_begin();
		$DBRes = db_query_params ('INSERT INTO plugin_gittracker_data_artifact (kind, group_artifact_id) VALUES (0, $1)',
			array ($Num));
		
		$HolderID= db_insertid($DBRes,'plugin_gittracker_data_artifact','id');
		
		if (!$DBRes || !$HolderID) {
			$return['Error']='Problems with Artifact $Num: '.db_error($DBRes);
			db_rollback();
		} else {
			$DBRes = db_query_params ('INSERT INTO plugin_gittracker_data_master (holder_id, git_date, log_text, file, prev_version, actual_version, author) VALUES ($1,$2,$3,$4,$5,$6,$7)',
			  array ($HolderID,
				 $Config['gitDate'],
				 $Config['Log'],
				 '',
				 $Config['PrevVersion'],
				 $Config['ActualVersion'],
				 $Config['UserName']));
			
			if(!$DBRes) {
				db_rollback();
			} else {
				db_commit();
			}
		}

	}
	if ($Rows > 1) {
		$return['Error'] .= "Unknown problem adding Tracker:$Num.";
	}
	return $return;
}

/**
 * Add a entry in the DataBase for a Tracker associated to a commit
 *
 * @param   array    $Config Config
 * @param   string   $GroupId The GroupId to insert it into
 * @param   string   $Num The tracker_id
 *
 * @return  arragetInvolvedArtifactsy    Returns 'check'=true if check passed, group, group_id
 */
function addTaskLog($Config, $GroupId, $Num)
{
	$return = array();
	$Result = db_query_params ('SELECT * from project_task,project_group_list WHERE project_task.group_project_id=project_group_list.group_project_id AND project_task.project_task_id=$1 AND project_group_list.group_id=$2',
	   array ($Num, $GroupId));
	
	$Rows = db_numrows($Result);
	
	if ($Rows == 0) {
		$return['Error'] .= "Task:$Num Not Found.";
	}
	
	if ($Rows == 1) {
		db_begin();
		$DBRes = db_query_params ('INSERT INTO plugin_gittracker_data_artifact (kind, project_task_id) VALUES (0, $1)',
			array ($Num));
		
		$HolderID= db_insertid($DBRes,'plugin_gittracker_data_artifact','id');
		
		if (!$DBRes || !$HolderID) {
			$return['Error']='Problems with Task $Num: '.db_error($DBRes);
			db_rollback();
		} else {
			$DBRes = db_query_params ('INSERT INTO plugin_gittracker_data_master (holder_id, git_date, log_text, file, prev_version, actual_version, author) VALUES ($1,$2,$3,$4,$5,$6,$7)',
				array ($HolderID,
					$Config['gitDate'],
					$Config['Log'],
					'',
					$Config['PrevVersion'],
					$Config['ActualVersion'],
					$Config['UserName']));
			
			if(!$DBRes) {
				db_rollback();
			} else {
				db_commit();
			}
		}
	}
	
	if ($Rows > 1) {
		$return['Error'] .= "Unknown problem adding Task:$Num.";
	}
	return $return;
}


exit (0);

// Local Variables:
// mode: php
// c-file-style: "bsd"
// End:

?>
